/* $Cambridge: hermes/src/prayer/prayer/cmd_copy.c,v 1.3 2008/04/09 16:03:45 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void
generate_favourite_folder_line(struct session *session,
                               struct pool *pool,
                               struct buffer *b, char *folder, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color;
    char *name = string_canon_path_encode(pool, "", folder);

    color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td>");
    html_icon(session, b, "blank.gif", "&nbsp;&nbsp;&nbsp;&nbsp;");
    html_session_bprintf(session, b, "<a href=\"", "\">", "copy_msg/%s", name);
    html_icon(session, b, "dir.gif", "[mailbox]");

    bputs(b, "</a>&nbsp;" CRLF);
    html_session_bprintf(session, b, "<a href=\"", "\">", "copy_msg/%s", name);
    html_quote_string(b, utf8_from_imaputf7(pool, folder));
    bputs(b, "</a>");
    bputs(b, "</td></tr>"CRLF);

}

static void
generate_folder_line(struct session *session, struct pool *pool,
                     struct buffer *b, struct folderitem *fi,
                     int depth, BOOL shaded)
{
    char *folder  = fi->name;
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *name = string_canon_encode(pool, folder);
    char *shortname;

    if ((shortname=strrchr(folder, session->hiersep[0])))
        shortname++;
    else
        shortname = folder;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td nowrap>");

    while (depth > 0) {
        html_icon(session, b, "blank.gif", "&nbsp;&nbsp;&nbsp;&nbsp;");
        depth--;
    }

    if (fi->haschildren) {
        html_session_bprintf(session, b, "<a href=\"", "\">",
                             "copy/toggle/%s", name);
        if (fi->expanded)
            html_icon(session, b, "down.gif", "[Collapse]");
        else
            html_icon(session, b, "right.gif", "[Expand]");
        bputs(b, "</a>&nbsp;");
    } else 
        html_icon(session, b, "blank.gif", "&nbsp;&nbsp;&nbsp;&nbsp;");

    if (!fi->noselect) {
        html_session_bprintf(session, b,
                             "<a href=\"", "\">" CRLF, "copy_msg/%s",
                             name);
        html_icon(session, b, "dir.gif", "[mailbox]");
        bputs(b, "</a>&nbsp;");

        html_session_bprintf(session, b,
                             "<a href=\"", "\">" CRLF, "copy_msg/%s",
                             name);
        html_quote_string(b, utf8_from_imaputf7(pool, shortname));
        bputs(b, "</a>");
    } else {
        html_icon(session, b, "blank.gif", "&nbsp;&nbsp;&nbsp;&nbsp;");
        bputs(b, "&nbsp;");
        html_quote_string(b, utf8_from_imaputf7(pool, shortname));
    }
    bputs(b, "</td>"CRLF);
    bprintf(b, "</tr>" CRLF);
}

/* ====================================================================== */

static void
html_folderlist_showtree(struct session *session,
                         struct pool *pool, struct buffer *b,
                         struct folderitem *fi, int depth, BOOL *shaded)
{
    struct prefs *prefs = session->options->prefs;
    BOOL suppress_dotfiles = prefs->suppress_dotfiles;

    while (fi) {
        if (!suppress_dotfiles || fi->name[0] != '.') {
            generate_folder_line(session, pool, b, fi, depth, *shaded);

            *shaded = (*shaded) ? NIL : T;

            if (fi->child && fi->expanded)
                html_folderlist_showtree(session, pool, b, fi->child,
                                         depth+1, shaded);
        }
        fi = fi->next;
    }
}

static void
folders_toolbar(struct session *session,
                struct pool *pool, struct buffer *b,
                struct folderlist *fl)
{
    struct prefs  *prefs  = session->options->prefs;
    struct config *config = session->config;
    struct config_theme *theme = session->theme_main;

    html_form_start(session, b, "GET", "create/copy");
    bputs(b, "<table width=\"100%\">");
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td><table><tr><td>" CRLF);

    if (config->dualuse) {
        bputs(b, "Create mailbox under " CRLF);
    } else {
        bputs(b, "Create" CRLF);
        bputs(b, "<select name=\"type\" size=1>" CRLF);
        bputs(b, "<option value=\"mailbox\" selected>mailbox</option>" CRLF);
        bputs(b, "<option value=\"directory\">directory</option>" CRLF);
        bputs(b, "</select>" CRLF);
        bputs(b, "under ");
    }
    bputs(b, "<select name=\"parent\" size=1>" CRLF);
    if (prefs->maildir && prefs->maildir[0]) {
        bprintf(b, "<option value=\"%s\">",
                string_canon_encode(pool, prefs->maildir));
        html_quote_string(b, prefs->maildir);
        bputs(b, " &lt;Root&gt;</option>" CRLF);
    } else if (!strcmp(session->personal_hierarchy, ""))
        bputs(b, "<option value=\"\">&lt;Root&gt;</option>" CRLF);
    folderlist_showdirs_select(fl, prefs->suppress_dotfiles, pool, b, NIL);
    bputs(b, "</select>" CRLF);
    bputs(b, "with name" CRLF);
    bputs(b, "<input name=\"name\">" CRLF);

    bputs(b,
          "<input type=\"submit\" name=\"create\" value=\"Create\">" CRLF);
    bputs(b, "</td></tr></table></td>" CRLF);
    bputs(b, "<td align=\"right\"><table><tr><td>" CRLF);
    html_textlink_session_table(session, b, session->copy_parent_cmd,
                                "Cancel");
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "dir_check/copy", "Refresh");
    bputs(b, "</tr></table>" CRLF);
    bputs(b, "</td></tr></table>" CRLF);
    html_form_end(b);
}


static void
html_folderlist(struct session *session,
                struct pool *pool,
                struct buffer *b, struct folderlist *fl,
                unsigned long count)
{
    struct favourite_list *fav = session->options->favourite_list;
    struct list_item *li;
    BOOL shaded = NIL;

    /* Title Bar */
    if (count != 1) {
        html_start(session, b,
                   pool_printf(pool, "Copy %lu Messages", count));
        html_banner_toolbar(session, b, "copy", 0);

        /* Title line */
        bprintf(b,
                "<h2 align=\"center\">Copy %lu Messages to folder...</h2>"
                CRLF, count);
    } else {
        html_start(session, b, "Copy message");
        html_banner_toolbar(session, b, "copy", 0);

        /* Title line */
        bprintf(b,
                "<h2 align=\"center\">Copy message to folder...</h2>"
                CRLF);
    }

    folders_toolbar(session, pool, b, fl);

    /* Start table for mailfolder listing */
    bputs(b, "<table width=\"100%\">");

    if (list_length(fav->list) > 0L) {
        /* Replace td with something which stops silly spacing */
        bputs(b, "<tr><td>Favourite mail folders:</td>" CRLF);
        bputs(b, "</tr>" CRLF);

        for (li = fav->list->head; li; li = li->next) {
            generate_favourite_folder_line(session, pool, b, li->name,
                                           shaded);
            shaded = (shaded) ? NIL : T;
        }

        bputs(b, "<tr><td>Normal folder listing:</td>" CRLF);
        bputs(b, "</tr>" CRLF);
    }

    html_folderlist_showtree(session, pool, b, fl->tree, 0, &shaded);

    /* End table for mailfolder listing */
    bprintf(b, "</table><br>" CRLF);

    /* Horizontal rule */
    bprintf(b, "<hr>" CRLF);

    folders_toolbar(session, pool, b, fl);
    html_end(b);
}

/* ====================================================================== */

void cmd_copy(struct session *session)
{
    struct request *request = session->request;
    struct options *options = session->options;
    struct prefs *prefs = options->prefs;
    struct pool *pool = request->pool;
    struct buffer *b = request->write_buffer;
    MAILSTREAM *stream = session->stream;
    unsigned long msgno, msgno_uid;
    unsigned long count = 1;
    struct folderitem *fi;
    char *name;

    if ((request->argc == 3) && !strcmp(request->argv[1], "toggle")) {
        name = string_canon_decode(request->argv[2]);
        fi   = folderlist_lookup(folderlist_fetch(session), name);
        if (fi) {
            if (fi->expanded) {
                session_message(session, "Collapsed \"%s\"", name);
                fi->expanded = NIL;
            } else {
                folderlist_expand(session, fi);
                session_message(session, "Expanded \"%s\"", name);
                fi->expanded = T;
            }
        }
    } else if ((request->argc > 1) && !strcmp(request->argv[1], "aggregate")) {
        if (prefs->use_mark_persist)
            count = msgmap_marked_count(session->zm);
        else
            count = msgmap_tmp_marked_count(session->zm);

        if (count > 1)
            session_message(session, "Copying %lu marked messages", count);
        else if (count == 1)
            session_message(session, "Copying single marked message");
        else {
            session_message(session, "No marked messages to copy");
            session_redirect(session, request, "list");
            return;
        }

        session->aggregate = T;
    } else if (request->argc > 2) {
        msgno = atoi(request->argv[1]);
        msgno_uid = atoi(request->argv[2]);

        if (!(msgno = stream_check_uid(session, stream, msgno, msgno_uid))) {
            session_redirect(session, request, "restart");
            return;
        }

        session->current = msgno;
        session->aggregate = NIL;
        session_message(session, "Copying message %lu", msgno);
    } else if (session->aggregate)
        count = msgmap_marked_count(session->zm);
    else
        count = 1;

    html_folderlist(session, pool, b, folderlist_fetch(session), count);

    response_html(request, 200);
}
