/* $Cambridge: hermes/src/prayer/prayer/cmd_folders.c,v 1.6 2008/04/09 16:03:45 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void
generate_folder_line(struct session *session, struct pool *pool,
                     struct buffer *b, struct folderitem *fi,
                     int depth, BOOL shaded)
{
    char *folder  = fi->name;
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *name = string_canon_encode(pool, folder);
    char *shortname;

    if ((shortname=strrchr(folder, session->hiersep[0])))
        shortname++;
    else
        shortname = folder;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    if (!strcmp(fi->name, "INBOX")) {
        bputs(b, "<td width=\"50%\" nowrap>");
    } else {
        bputs(b, "<td nowrap>");

    }

    while (depth > 0) {
        html_icon(session, b, "blank.gif", "&nbsp;&nbsp;&nbsp;&nbsp;");
        depth--;
    }

    if (fi->haschildren) {
        html_session_bprintf(session, b, "<a href=\"", "\">",
                             "folders/toggle/%s", name);
        if (fi->expanded)
            html_icon(session, b, "down.gif", "[Collapse]");
        else
            html_icon(session, b, "right.gif", "[Expand]");
        bputs(b, "</a>&nbsp;");
    } else 
        html_icon(session, b, "blank.gif", "&nbsp;&nbsp;&nbsp;&nbsp;");

    if (!fi->noselect) {
        html_session_bprintf(session, b, "<a href=\"", "\">",
                             "change/%s", name);
        html_icon(session, b, "dir.gif", "[mailbox]");
        bputs(b, "</a>&nbsp;");
        html_session_bprintf(session, b, "<a href=\"", "\">",
                             "change/%s", name);
        html_quote_string(b, utf8_from_imaputf7(pool, shortname));
        bputs(b, "</a>");
    } else {
        html_icon(session, b, "blank.gif", "&nbsp;&nbsp;&nbsp;&nbsp;");
        bputs(b, "&nbsp;");
        html_quote_string(b, utf8_from_imaputf7(pool, shortname));
    }
    bputs(b, "</td>"CRLF);

    if (!strcmp(fi->name, "INBOX")) {
            bputs(b, "<td width=\"25%\">&nbsp;</td>" CRLF);
            bputs(b, "<td width=\"25%\">&nbsp;</td>" CRLF);
    } else {
        html_textlink_mailbox_table(session, b, "rename", name, "Rename");
        html_textlink_mailbox_table(session, b, "rm/mailbox", name, "Delete");
    }
    bprintf(b, "</tr>" CRLF);
}

/* ====================================================================== */

static void
html_folderlist_showtree(struct session *session,
                         struct pool *pool, struct buffer *b,
                         struct folderitem *fi, int depth, BOOL *shaded)
{
    struct prefs *prefs = session->options->prefs;
    BOOL suppress_dotfiles = prefs->suppress_dotfiles;

    while (fi) {
        if (!suppress_dotfiles || fi->name[0] != '.') {
            generate_folder_line(session, pool, b, fi, depth, *shaded);

            *shaded = (*shaded) ? NIL : T;

            if (fi->child && fi->expanded)
                html_folderlist_showtree(session, pool, b, fi->child,
                                         depth+1, shaded);
        }
        fi = fi->next;
    }
}

static void
folders_toolbar(struct session *session,
                struct pool *pool, struct buffer *b,
                struct folderlist *fl)
{
    struct prefs  *prefs  = session->options->prefs;
    struct config *config = session->config;
    struct config_theme *theme = session->theme_main;

    html_form_start(session, b, "GET", "create/folders");
    bputs(b, "<table width=\"100%\">");
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td><table><tr><td>" CRLF);

    if (config->dualuse) {
        bputs(b, "Create mailbox under " CRLF);
    } else {
        bputs(b, "Create" CRLF);
        bputs(b, "<select name=\"type\" size=1>" CRLF);
        bputs(b, "<option value=\"mailbox\" selected>mailbox</option>" CRLF);
        bputs(b, "<option value=\"directory\">directory</option>" CRLF);
        bputs(b, "</select>" CRLF);
        bputs(b, "under ");
    }
    bputs(b, "<select name=\"parent\" size=1>" CRLF);
    if (prefs->maildir && prefs->maildir[0]) {
        bprintf(b, "<option value=\"%s\">",
                string_canon_encode(pool, prefs->maildir));
        html_quote_string(b, prefs->maildir);
        bputs(b, " &lt;Root&gt;</option>" CRLF);
    } else if (!strcmp(session->personal_hierarchy, ""))
        bputs(b, "<option value=\"\">&lt;Root&gt;</option>" CRLF);
    folderlist_showdirs_select(fl, prefs->suppress_dotfiles, pool, b, NIL);
    bputs(b, "</select>" CRLF);
    bputs(b, "with name" CRLF);
    bputs(b, "<input name=\"name\">" CRLF);

    bputs(b,
          "<input type=\"submit\" name=\"create\" value=\"Create\">" CRLF);
    bputs(b, "</td></tr></table></td>" CRLF);
    bputs(b, "<td align=\"right\"><table><tr><td>" CRLF);
    html_textlink_session_table(session, b, "dir_check/folders",
                                "Refresh");
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "transfer",
                                "Transfer Folders");
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "favourites",
                                "Favourite Folders");
    bputs(b, "</tr></table></td></tr></table>" CRLF);
    html_form_end(b);
}

static void
html_folderlist(struct session *session,
                struct pool *pool, struct buffer *b,
                struct folderlist *fl)
{
    BOOL shaded = NIL;

    html_start(session, b, "Mail Folders");

    html_banner_toolbar(session, b, "folders", 0);

    /* Title line */
    bprintf(b, "<h2 align=\"center\">Mail Folders</h2>" CRLF);

    folders_toolbar(session, pool, b, fl);

    /* Start table for mailfolder listing */
    bputs(b, "<table width=\"100%\">" CRLF);

    html_folderlist_showtree(session, pool, b, fl->tree, 0, &shaded);

    /* End table for mailfolder listing */
    bprintf(b, "</table>" CRLF);

    folders_toolbar(session, pool, b, fl);
    html_end(b);
}

/* ====================================================================== */

void cmd_folders(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct folderitem *fi;
    char *name;

    /* Record last command for compose/cancel and friends */
    if (!session->draft->have_draft)
        session->compose_parent_cmd = "folders";

    if ((request->argc == 3) && !strcmp(request->argv[1], "toggle")) {
        name = string_canon_decode(request->argv[2]);
        fi   = folderlist_lookup(folderlist_fetch(session), name);
        if (fi) {
            if (fi->expanded) {
                session_message(session, "Collapsed \"%s\"", name);
                fi->expanded = NIL;
            } else {
                folderlist_expand(session, fi);
                session_message(session, "Expanded \"%s\"", name);
                fi->expanded = T;
            }
        }
    }

    /* Save options if anything changed */
    if (session->options->save)
        session_streams_save_options(session);

    html_folderlist(session, request->pool, b, folderlist_fetch(session));

    response_html(request, 200);
}
