/* $Cambridge: hermes/src/prayer/prayer/cmd_rename.c,v 1.2 2008/04/09 16:03:45 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"


static void
generate_folder_line(struct session *session, struct pool *pool,
                     struct buffer *b, struct folderitem *fi,
                     int depth, BOOL shaded)
{
    char *folder  = fi->name;
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *name = string_canon_encode(pool, folder);
    char *shortname;

    if ((shortname=strrchr(folder, session->hiersep[0])))
        shortname++;
    else
        shortname = folder;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td nowrap>");

    while (depth > 0) {
        html_icon(session, b, "blank.gif", "&nbsp;&nbsp;&nbsp;&nbsp;");
        depth--;
    }

    if (fi->haschildren) {
        html_session_bprintf(session, b, "<a href=\"", "\">",
                             "rename/toggle/%s", name);
        if (fi->expanded)
            html_icon(session, b, "down.gif", "[Collapse]");
        else
            html_icon(session, b, "right.gif", "[Expand]");
        bputs(b, "</a>");
    } else
        html_icon(session, b, "blank.gif", "&nbsp;&nbsp;&nbsp;&nbsp;");

    if (!fi->noselect) 
        html_icon(session, b, "dir.gif", "[mailbox]");
    else
        html_icon(session, b, "blank.gif", "&nbsp;&nbsp;&nbsp;&nbsp;");

    bputs(b, "&nbsp;");
    html_quote_string(b, utf8_from_imaputf7(pool, shortname));
    bputs(b, "</td>" CRLF);

    bputs(b, "<td></td><td></td>");
    bprintf(b, "</tr>" CRLF);
}

static void
html_folderlist_showtree(struct session *session,
                         struct pool *pool, struct buffer *b,
                         struct folderitem *fi, int depth, BOOL *shaded)
{
    struct prefs *prefs = session->options->prefs;
    BOOL suppress_dotfiles = prefs->suppress_dotfiles;

    while (fi) {
        if (!suppress_dotfiles || fi->name[0] != '.') {
            generate_folder_line(session, pool, b, fi, depth, *shaded);
        
            *shaded = (*shaded) ? NIL : T;

            if (fi->child && fi->expanded)
                html_folderlist_showtree(session, pool, b, fi->child,
                                         depth+1, shaded);
        }
        fi = fi->next;
    }
}

static void
folders_toolbar(struct session *session,
                struct pool *pool, struct buffer *b,
                struct folderlist *fl)
{
    struct prefs *prefs = session->options->prefs;
    char *oldname = session->rename_foldername;
    struct config_theme *theme = session->theme_main;
    char *parent = NIL, *s;

    if (oldname && (s=strrchr(oldname, session->hiersep[0]))) {
        *s = '\0';
        parent = pool_strdup(pool, oldname);
        *s = session->hiersep[0];
    }

    html_form_start(session, b, "POST", "rename_item");
    bputs(b, "<table width=\"100%\">");
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td>" CRLF);

    bputs(b, "Rename under ");
    bputs(b, "<select name=\"parent\" size=1>" CRLF);
    if (prefs->maildir && prefs->maildir[0]) {
        bprintf(b, "<option value=\"%s\"%s>",
                string_canon_encode(pool, prefs->maildir),
                (parent) ? "" : " selected");
        html_quote_string(b, prefs->maildir);
        bputs(b, " &lt;Root&gt;</option>" CRLF);
    } else if (!strcmp(session->personal_hierarchy, "")) {
        bprintf(b, "<option value=\"\"%s>&lt;Root&gt;</option>" CRLF,
                (parent) ? "" : " selected");
    }
    folderlist_showdirs_select(fl, prefs->suppress_dotfiles, pool, b, parent);
    bputs(b, "</select>" CRLF);
    bputs(b, " as" CRLF);
    html_form_field(b, "new name", "name", "", 16);
    html_form_submit(b, "submit", "Rename");
    html_form_submit(b, "cancel", "Cancel");
    bputs(b, "</td></tr></table>" CRLF);
    html_form_end(b);
}

static void
html_folderlist(struct session *session,
                struct pool *pool,
                struct buffer *b, struct folderlist *fl)
{
    BOOL shaded = NIL;
    char *oldname = session->rename_foldername;

    /* Title Bar */
    html_start(session, b, "Rename mail folder");
    html_banner_toolbar(session, b, "rename", 0);

    /* Title line */
    if (oldname) {
        bputs(b, "<h2 align=\"center\">Rename folder \"");
        html_quote_string(b, utf8_from_imaputf7(pool, oldname));
        bputs(b, "\" to ...</h2>" CRLF);
    }

    folders_toolbar(session, pool, b, fl);

    /* Start table for mailfolder listing */
    bputs(b, "<table width=\"100%\">" CRLF);

    html_folderlist_showtree(session, pool, b, fl->tree, 0, &shaded);

    /* End table for mailfolder listing */
    bprintf(b, "</table>" CRLF);

    /* Horizontal rule */
    bprintf(b, "<hr>" CRLF);

    folders_toolbar(session, pool, b, fl);
    html_end(b);
}

/* ====================================================================== */

void cmd_rename(struct session *session)
{
    struct request *request = session->request;
    struct pool *pool = request->pool;
    struct buffer *b = request->write_buffer;
    struct folderitem *fi;
    char *name;

    if ((request->argc == 3) && !strcmp(request->argv[1], "toggle")) {
        name = string_canon_decode(request->argv[2]);
        fi   = folderlist_lookup(folderlist_fetch(session), name);
        if (fi) {
            if (fi->expanded) {
                session_message(session, "Collapsed \"%s\"", name);
                fi->expanded = NIL;
            } else {
                folderlist_expand(session, fi);
                session_message(session, "Expanded \"%s\"", name);
                fi->expanded = T;
            }
        }
    } else if (request->argc == 2) {
        string_strdup(&session->rename_foldername, request->argv[1]);
        string_canon_decode(session->rename_foldername);
    }

    html_folderlist(session, pool, b, folderlist_fetch(session));

    response_html(request, 200);
}
