/* $Cambridge: hermes/src/prayer/prayer/cmd_upload_select.c,v 1.3 2008/04/09 16:03:45 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void
generate_folder_line(struct session *session, struct pool *pool,
                     struct buffer *b, struct folderitem *fi,
                     int depth, BOOL shaded)
{
    char *folder  = fi->name;
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *name = string_canon_encode(pool, folder);
    char *shortname;

    if ((shortname=strrchr(folder, session->hiersep[0])))
        shortname++;
    else
        shortname = folder;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td nowrap>");

    while (depth > 0) {
        html_icon(session, b, "blank.gif", "&nbsp;&nbsp;&nbsp;&nbsp;");
        depth--;
    }

    if (fi->haschildren) {
        html_session_bprintf(session, b, "<a href=\"", "\">",
                             "upload_select/toggle/%s", name);
        if (fi->expanded)
            html_icon(session, b, "down.gif", "[Collapse]");
        else
            html_icon(session, b, "right.gif", "[Expand]");
        bputs(b, "</a>");
    } else
        html_icon(session, b, "blank.gif", "&nbsp;&nbsp;&nbsp;&nbsp;");

    if (!fi->noselect) 
        html_icon(session, b, "dir.gif", "[mailbox]");
    else
        html_icon(session, b, "blank.gif", "&nbsp;&nbsp;&nbsp;&nbsp;");

    bputs(b, "&nbsp;");
    html_quote_string(b, utf8_from_imaputf7(pool, shortname));
    bputs(b, "</td>" CRLF);

    html_textlink_mailbox_table(session, b, "upload_xfer", folder,
                                "Append");

    bprintf(b, "</tr>" CRLF);
}

static void
html_folderlist_showtree(struct session *session,
                         struct pool *pool, struct buffer *b,
                         struct folderitem *fi, int depth, BOOL *shaded)
{
    struct prefs *prefs = session->options->prefs;
    BOOL suppress_dotfiles = prefs->suppress_dotfiles;

    while (fi) {
        if (!suppress_dotfiles || fi->name[0] != '.') {
            generate_folder_line(session, pool, b, fi, depth, *shaded);
        
            *shaded = (*shaded) ? NIL : T;
            
            if (fi->child && fi->expanded)
                html_folderlist_showtree(session, pool, b, fi->child,
                                         depth+1, shaded);
        }
        fi = fi->next;
    }
}

/* ====================================================================== */

static void
html_folderlist(struct session *session,
                struct pool *pool, struct buffer *b,
                struct folderlist *fl)
{
    struct prefs *prefs = session->options->prefs;
    BOOL shaded = NIL;

    /* Title bar */
    html_start(session, b, "Select Target Mailfolder");
    html_banner_toolbar(session, b, "upload_select", 0);

    /* Title line */
    bputs(b, "<h2 align=\"center\">Upload local folder \"");
    if (session->upload_name)
        html_quote_string(b, session->upload_name);
    bputs(b, "\" as folder</h2>" CRLF);

    /* Horizontal rule */
    bprintf(b, "<hr>" CRLF);

    /* Start table for mailfolder listing */
    bputs(b, "<table width=\"100%\">");

    html_folderlist_showtree(session, pool, b, fl->tree, 0, &shaded);

    /* End table for mailfolder listing */
    bputs(b, "</table>" CRLF);
    bputs(b, "<hr>" CRLF);

    html_form_start(session, b, "GET", "upload_xfer");

    bputs(b, "Upload folder under ");
    bputs(b, "<select name=\"parent\" size=1>" CRLF);
    if (prefs->maildir && prefs->maildir[0]) {
        bprintf(b, "<option value=\"%s\" selected>",
                string_canon_encode(pool, prefs->maildir));
        html_quote_string(b, prefs->maildir);
        bputs(b, " &lt;Root&gt;</option>" CRLF);
    } else if (!strcmp(session->personal_hierarchy, ""))
        bputs(b, "<option value=\"\" selected>&lt;Root&gt;</option>" CRLF);
    folderlist_showdirs_select(fl, prefs->suppress_dotfiles, pool, b, NIL);
    bputs(b, "</select>" CRLF);
    html_form_field(b, "with name:", "name", session->upload_name, 40);

    html_form_submit(b, "create", "Create");
    html_form_submit(b, "append", "Append");
    html_form_submit(b, "cancel", "Cancel");
    html_form_end(b);
    html_end(b);
}

/* ====================================================================== */

static BOOL upload(struct session *session,
                   char *name, char *data, unsigned long len)
{
    struct config *config = session->config;
    struct request *request = session->request;
    char *tmpfname;
    FILE *tmpfile;
    int fd;

    tmpfname = pool_printf(request->pool, "%s/%s:%lu:XXXXXX",
                           config->tmp_dir, session->username,
                           (unsigned long) getpid());

    if ((fd = mkstemp(tmpfname)) < 0)
        return (NIL);

    /* Transfer data to temporary file */
    if ((tmpfile = fdopen(fd, "w")) == NIL) {
        session_message(session, "Failed to open temporary file");
        return (NIL);
    }

    while (len > 0) {
        fputc((*data) ? (*data) : ' ', tmpfile);

        data++;
        len--;
    }
    fclose(tmpfile);

    if (session->upload_file) {
        unlink(session->upload_file);
        string_free((void **) &session->upload_file);
    }
    session->upload_file = pool_strdup(NIL, tmpfname);
    return (T);
}

/* ====================================================================== */

BOOL
process_upload_request(struct session * session, struct request * request)
{
    char *s, *disp;
    char *start, *end;
    struct assoc *hdrs = assoc_create(request->pool, 16, T);
    char *name = NIL;
    unsigned long len;

    request_decode_post_multipart(request, hdrs, &start, &end);
    *end = '\0';

    if (end > start)
        len = (char *) end - (char *) start;
    else
        len = 0;

    if ((disp = assoc_lookup(hdrs, "content-disposition"))) {
        for (s = disp; *s; s++) {
            if (!strncasecmp(s, "filename=\"", strlen("filename=\""))) {
                name = s = s + strlen("filename=\"");
                while (*s && (*s != '"'))
                    s++;
                *s = '\0';
                break;
            }
        }
    }
    if (name && name[0]) {
        if ((s = strrchr(name, '/')))
            name = s + 1;
        else if ((s = strrchr(name, '\\')))
            name = s + 1;
    } else
        name = "Upload";

    if (!upload(session, name, start, len))
        return (NIL);

    session_message(session, "Copied mailbox to halfway point");
    session->upload_name = pool_strdup(NIL, name);
    return (T);
}

void cmd_upload_select(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct folderitem *fi;
    char *name;

    if (request->method == POST) {
        if (!process_upload_request(session, request)) {
            session_redirect(session, request, "restart");
            return;
        }
    }

    if ((request->argc == 3) && !strcmp(request->argv[1], "toggle")) {
        name = string_canon_decode(request->argv[2]);
        fi   = folderlist_lookup(folderlist_fetch(session), name);
        if (fi) {
            if (fi->expanded) {
                session_message(session, "Collapsed \"%s\"", name);
                fi->expanded = NIL;
            } else {
                folderlist_expand(session, fi);
                session_message(session, "Expanded \"%s\"", name);
                fi->expanded = T;
            }
        }
    }

    html_folderlist(session, request->pool, b, folderlist_fetch(session));
    response_html(request, 200);
}
